#!/bin/bash
UL="\033[4m"
EUL="\033[0m"
USAGE="Usage: `basename $0` ${UL}old_storage${EUL} ${UL}new_storage${EUL}
"

shift $((OPTIND-1))

if [ $# -ne 2 ]; then
	echo -e "Need names for both old and new storage"
	echo -e "${USAGE}"
	exit 1
fi

LOGFILE=/usr/bp/logs.dir/migrate_S3_data.log
OLD_STORAGE=$1
NEW_STORAGE=$2
sleep_interval=5

function finish {
	psql -c "UPDATE bp.schedules SET enabled = t WHERE schedule_id in (SELECT schedule_id FROM bp.archive_schedule_profiles WHERE profile_id in (SELECT profile_id from bp.archive_profiles WHERE target in ('${OLD_STORAGE}','${NEW_STORAGE}'))" &>/dev/null
}

echo "This utility is for moving Cold Backup Copy data from an old AWS bucket using s3ql 1 to a new bucket using s3ql 2."
echo "Moving Cold Backup Copy data from ${OLD_STORAGE} to ${NEW_STORAGE}."
read -p "Is this correct?[N/y] " -r resp
if [[ ! $resp =~ ^[Yy]$ ]]; then
	exit 1
fi
echo

echo "Starting migrations" > ${LOGFILE}

trap finish EXIT
psql -c "UPDATE bp.schedules SET enabled = f WHERE schedule_id in (SELECT schedule_id FROM bp.archive_schedule_profiles WHERE profile_id in (SELECT profile_id from bp.archive_profiles WHERE target in ('${OLD_STORAGE}','${NEW_STORAGE}'))" &>/dev/null

# MOUNT THE BUCKETS
if mount | grep -wq "${OLD_STORAGE}"; then
	echo "${OLD_STORAGE} already mounted"
else
	echo -n "Mounting ${OLD_STORAGE} ... "
	cloudadm -n connect ${OLD_STORAGE} &>/dev/null
	if [ ! $? -eq 0 ]; then
		echo "Failed"
		echo "Failed to mount ${OLD_STORAGE}" >> ${LOGFILE}
		exit 1
	fi
	sleep ${sleep_interval}
	echo "Done"
fi

if mount | grep -wq "${NEW_STORAGE}"; then
	echo "${NEW_STORAGE} already mounted"
else
	echo -n "Mounting ${NEW_STORAGE} ... "
	cloudadm connect ${NEW_STORAGE} &>/dev/null
	if [ ! $? -eq 0 ]; then
		echo "Failed"
		echo "Failed to mount ${NEW_STORAGE}" >> ${LOGFILE}
		exit 1
	fi
	sleep ${sleep_interval}
	echo "Done"
fi
echo "Mounting finished." >> ${LOGFILE}

# CHECK FOR .asset FILE
if [ ! -f /backups/${NEW_STORAGE}/.asset ]; then
	cp -rfP /backups/${OLD_STORAGE}/.asset /backups/${NEW_STORAGE}
fi

# COPY ARCHIVE DATA
copy_success=1
if [ -d /backups/${OLD_STORAGE} -a -d /backups/${NEW_STORAGE} ]; then
	sets=`find /backups/${OLD_STORAGE}/ -maxdepth 1 -type d -name "????-??-??_??_??_??"`
	total=`find /backups/${OLD_STORAGE}/ -maxdepth 1 -type d -name "????-??-??_??_??_??" | wc -l`
	count=1
	if [ ${total} -eq 0 ]; then
		echo "No Cold Backup Copy sets found on ${OLD_STORAGE}."
		echo "No Cold Backup Copy sets found on ${OLD_STORAGE}." >> ${LOGFILE}
	else
		for dir in ${sets}; do
			echo -ne "\rCopying set ${count}/${total}"
			echo "Copying set ${count}/${total}" >> ${LOGFILE}
			if ! cp -frP ${dir} /backups/${NEW_STORAGE}; then
				echo " ... Failed"
				break
			fi
			if [ ${count} -eq ${total} ]; then
				copy_sucess=0
				echo " ... Done"
				echo "Copying finished" >> ${LOGFILE}
			fi
			((count++))
		done
	fi
else
	echo "Non-standard directory path, did mounting work?"
	echo "Non-standard directory path" >> ${LOGFILE}
	exit 1
fi

# CHANGE SERIAL IN DB
if [ ${copy_success} ]; then
	OLD_SERIAL=`cat /backups/${OLD_STORAGE}/.unitrends_serial`
	NEW_SERIAL=`cat /backups/${NEW_STORAGE}/.unitrends_serial`

	psql -c "UPDATE bp.archive_disks SET serial_no = '${NEW_SERIAL}' WHERE serial_no = '${OLD_SERIAL}'" &>/dev/null
	psql -c "SET session_replication_role = replica; UPDATE bp.archive_profiles SET target = '${NEW_STORAGE}' WHERE profile_id in (SELECT DISTINCT profile_id FROM bp.archive_sets WHERE archive_set_id in (SELECT archive_set_id FROM bp.archive_disks WHERE serial_no = '${NEW_SERIAL}')); SET session_replication_role = DEFAULT;" &>/dev/null

	# SYNC THE NEW BUCKET
	echo -n "Syncing ${NEW_STORAGE} ... "
	cloudadm sync ${NEW_STORAGE} &>/dev/null
	if [ ! $? -eq 0 ]; then
		echo "Returned non-zero"
		echo "${NEW_STORAGE} failed to sync" >> ${LOGFILE}
	else
		echo "Done"
		echo "${NEW_STORAGE} synced" >> ${LOGFILE}
	fi
	sleep ${sleep_interval}
fi

# UNMOUNT OLD BUCKET
if mount | grep -wq "${OLD_STORAGE}"; then
	echo -n "Unmounting ${OLD_STORAGE} ... "
	cloudadm -n disconnect ${OLD_STORAGE} &>/dev/null
	if [ ! $? -eq 0 ]; then
		echo "Failed"
		exit 1
	fi
	echo "Done"
fi

echo "Migration complete" >> ${LOGFILE}
